﻿'Dans ce troisième tutorial nous allons aussi charger un objet 3d mais à partir
'd'un fichier et lui appliquer de la lumière
Imports Tzu3D
Imports Tzu3D.TZGlobal
Imports System.ComponentModel

Public Class Form1
    Private Engine As TZEngine
    Private myScene As Scene.TZScene

    Private myCameraActor As Scene.TZActor
    Private myCamera As Tzu3D.Component.TZCamera

    Private myActor As Scene.TZActor
    Private myMesh As Tzu3D.Component.TZMesh

    Private Light As Tzu3D.Component.TZLight
    Private LightActor As Scene.TZActor
    Dim OldWindowState As FormWindowState

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        Engine = TZEngine.Instance
        Engine.Init(Me.Handle)
        Engine.Debug_Enable = True
        Engine.Show_Fps = True
        Engine.Debug_Enable = True
        Engine.Add_MediaPath(My.Application.Info.DirectoryPath & "\..\..\..\..\..\media\")

        Creating()

        Loading()

        Updating()

        Rendering()
    End Sub

    Private Sub Creating()
        myScene = New Scene.TZScene("myScene")
        myScene.Light_Ambient_Direction(eAmbient.Down) = Color.FromArgb(4, 4, 8)
        myScene.Light_Ambient_Direction(eAmbient.Middle) = Color.FromArgb(7, 7, 12)
        myScene.Light_Ambient_Direction(eAmbient.Up) = Color.FromArgb(81, 68, 0)

        myCamera = New Tzu3D.Component.TZCamera("myCamera")
        myCamera.Perspective(75, 16 / 9, 0.1, 50)
        myCamera.Projection_Mode = eCameraProjection.Perspective
        myCamera.Set_Mode(eCameraMode.First_Person)

        myCameraActor = New Scene.TZActor("myCameraActor")
        myCameraActor.Position = New Vector3(0, 4, -4)
        myCameraActor.Add_Component(myCamera)
        myScene.Add(myCameraActor)

        'chargement d'un objet 3d en indiquant le nom de son fichier
        myMesh = New Tzu3D.Component.TZMesh("Stone.obj", "stone")

        'ce fichier .obj a la particularité de ne pas possédé de materiaux associé dans un fichier .mtl
        'on va donc ajouter manuellement ses differentes textures
        Dim MeshMaterial As Texture.TZMaterial = myMesh.Material(0)
        MeshMaterial.Texture_Diffuse = New Texture.TZTexture("stone_albedo.png", "diffuse")
        MeshMaterial.Texture_Normals = New Texture.TZTexture("stone_normal.png", "normal")
        MeshMaterial.Texture_Specular = New Texture.TZTexture("stone_ao.png", "specular")

        'et indiquer et l'on utilise une NormalMap et activer également la lumière spéculaire
        MeshMaterial.Enable_NormalMap = True
        MeshMaterial.Enable_Specular = True
        MeshMaterial.CullMode = eCullMode.Front
        MeshMaterial.Shininess = 80
        'on oublie pas de donner le materiau à l'objet
        myMesh.Material(0) = MeshMaterial

        myActor = New Scene.TZActor("myActor")
        myActor.Position = New Vector3(0, 0, 0)
        myActor.Scale = New Vector3(2, 2, 2)
        myActor.Add_Component(myMesh)
        myScene.Add(myActor)

        'création d'une lumière directionnel de couleur blanche
        Light = New Tzu3D.Component.TZLight("myLight")
        Light.Type = eLight_Type.Directional
        Light.Diffuse = Color.White
        Light.Intensity = 1
        Light.Specular = Color.White

        'et on l'active
        Light.Enable = True

        'on créé un acteur qui va porter cette lumière
        LightActor = New Scene.TZActor("LightActor")
        LightActor.Add_Component(Light)
        LightActor.Direction = New Vector3(-0.25, -1, 0)
        myScene.Add(LightActor)
        OldWindowState = Me.WindowState
    End Sub

    Private Sub Loading()
        myScene.Load()

        Engine.Load()
    End Sub

    Private Sub Updating()
        myCamera.LookAt(New Vector3(0, 0, 0))
    End Sub

    Private Sub Rendering()
        Do While Engine.DoRender
            Engine.Render_Begin(Color.Black)

            Controller()

            myScene.Process()

            myScene.Render()

            Engine.Render_End()

            Application.DoEvents()
        Loop

        Engine.Dispose()

        Application.Exit()
    End Sub

    Private Sub Controller()
        'rotation de l'objet sur les 3 axes
        myActor.Rotate_Step(30 * Engine.DeltaTime, 30 * Engine.DeltaTime, 30 * Engine.DeltaTime)
    End Sub

    Private Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        If Engine.DoRender Then
            Engine.Quit()
            e.Cancel = True
        End If
    End Sub

    Private Sub Form1_ResizeEnd(sender As Object, e As EventArgs) Handles Me.ResizeEnd
        If Engine Is Nothing Then Exit Sub
        Engine.Resize_Windows()
    End Sub


    Private Sub Form1_SizeChanged(sender As Object, e As EventArgs) Handles Me.SizeChanged
        If Engine Is Nothing Then Exit Sub
        If Me.WindowState = FormWindowState.Maximized OrElse Me.WindowState = FormWindowState.Normal Then
            If OldWindowState <> Me.WindowState Then
                Engine.Resize_Windows()
                OldWindowState = Me.WindowState
            End If
        End If
    End Sub
End Class
