﻿'Dans ce premier tutoriel nous allons initialiser le moteur
'et écrire le fameux "Hello World" au milieu de la fenetre de rendu

'Importation du namespace Tzu3D
Imports Tzu3D
'On importe également la classe TZGlobal qui contient des fonctions utiles un peu partout
Imports Tzu3D.TZGlobal

Imports System.ComponentModel

Public Class Form1
    'Declaration de la classe TZEngine, qui est le coeur de Tzu3D
    Private Engine As TZEngine

    'Déclaration d'une classe TZSprite qui permet de dessiner des sprites 2d et 3d
    Private mySprite As Graphics.TZSprite

    'Déclaration d'une classe TZFont qui permet d'écrire du texte en 2d et 3d. (Pour du texte 3d il est préférable d'utiliser le composant TZText)
    Private myFont As Graphics.TZFont

    'Declaration d'une chaine de caractère où l'on va écrire notre "Hello World" en couleur
    Private myText As String

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        'On récupère l'instance de TZEngine
        Engine = TZEngine.Instance

        'Puis on initialise le moteur en mode fenêtré en indiquant le handle du controle où s'effectuera le rendu
        Engine.Init(Me.Handle)

        'On indique que l'on souhaite afficher le nombre d'images par seconde
        Engine.Show_Fps = True

        'Afin de bien organiser notre code, nous allons utiliser 4 étapes avant de permetre le rendu

        'La création: c'est ici que l'on créé tout ce que l'on souhaite afficher lors du rendu
        Creating()

        'Le chargement: c'est ici que le moteur charge tout les composants qui ont été créés. 
        Loading()

        'Mise à jour: Après le chargement, il est parfois nécessaire de faire quelques modifications avant le rendu
        Updating()

        'Rendu: c'est ici que sera dessiné tout ce que l'on souhaite dans une boucle de rendu
        Rendering()
    End Sub

    Private Sub Creating()
        'Nous allons faire le rendu d'un rectangle "ombré" et écrire par dessus du texte en couleur
        'Nous avont donc besoin de créer une classe TZSprite ainsi qu'une classe TZFont
        mySprite = New Graphics.TZSprite(50)
        myFont = New Graphics.TZFont("Comic Sans MS", 35, FontStyle.Bold, eTextEffect.None)

        'Formatage d'une chaine de caractère contenant le mot "HELLO" écrit en blue, WORLD écrit en rouge et le point d'exclamation en vert
        myText = String.Format("{0}HELLO {1}WORLD {2}!", Colors.ColorString(Color.Blue), Colors.ColorString(Color.Red), Colors.ColorString(Color.Green))
    End Sub

    Private Sub Loading()
        'Chargements des compostants nécessaire au rendu en interne du moteur
        Engine.Load()
    End Sub

    Private Sub Updating()
        'pour ce simple rendu nous n'avons rien à modifier
    End Sub

    Private Sub Rendering()

        'Début de la boucle de rendu, tant que DoRender est vrai
        Do While Engine.DoRender

            'Debut du rendu avec une couleur de fond bleu-violet
            Engine.Render_Begin(Color.BlueViolet)

            'L'ordre du rendu est important, spécialement en 2d, chaque sprite est dessiné par dessus le précédent

            'On commence par dessiner un grand rectangle noir qui fera office de cadre
            mySprite.Draw(New Vector4(0, 0, 1, 1), New Vector4(150 - 5, 150 - 5, 500 + 10, 300 + 10), Color.Black)

            'Par dessus, on dessine un rectangle orange, plus petit, afin de ne laisser qu'une légère bordure noir tout autour
            mySprite.Draw(New Vector4(0, 0, 1, 1), New Vector4(150, 150, 500, 300), Color.Orange)

            'Et pour terminer, on rajoute 2 lignes: vertical et horizontal, pour simuler un ombrage du cadre
            mySprite.DrawLine(New Vector4(0, 0, 1, 1), New Vector2(150 - 5, 150 + 300 + 7.5), New Vector2(150 + 500 + 5, 150 + 300 + 7.5), 5, Color.FromArgb(64, Color.Black), 1)
            mySprite.DrawLine(New Vector4(0, 0, 1, 1), New Vector2(150 + 500 + 7.5, 150 - 5), New Vector2(150 + 500 + 7.5, 150 + 300 + 10), 5, Color.FromArgb(64, Color.Black), 1)

            'On indique que nos dessins doivent être dessiné immédiatement à l'écran
            mySprite.Render()

            'Et enfin, nous écrivons par dessus notre texte.
            myFont.Write_Text(myText, New Vector2(180, 270), Color.White, 1.0F)
            myFont.RenderText()

            'Fin de la boucle de rendu
            Engine.Render_End()

            'On laisse le temps à windows de traiter ses messages avant de recommencer la boucle de rendu
            Application.DoEvents()
        Loop

        'Si la boucle de rendu est terminé, alors on décharge tout les composants du moteur afin de quitter proprement
        Engine.Dispose()

        Application.Exit()
    End Sub

    Private Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        'si la fenêtre est fermé, on indique au moteur que l'on souhaite sortir de la boucle de rendu
        If Engine.DoRender Then
            Engine.Quit()
            e.Cancel = True
        End If
    End Sub
End Class
