﻿'Dans ce tutoriel nous allons afficher un cube tournant sur lui même et lui affecter une texture

Imports Tzu3D
Imports System.ComponentModel

Public Class Form1
    Private Engine As TZEngine

    'Afin de dessiner des objets, nous avons besoin d'une scene qui va etre alimenter par des acteurs qui eux meme seront alimenté par des composants
    Private myScene As Scene.TZScene

    'Declaration d'un acteur qui contiendra un composant camera
    Private myCameraActor As Scene.TZActor
    Private myCamera As Tzu3D.Component.TZCamera

    'déclaration d'un acteur qui contiendra un composant Mesh (un objet 3d)
    Private myActor As Scene.TZActor
    Private myMesh As Tzu3D.Component.TZMesh

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Engine = TZEngine.Instance

        Engine.Init(Me.Handle)

        Engine.Show_Fps = True

        'on indique le dossier contenant nos media (objets, textures, sons, etc...)
        Engine.Add_MediaPath(My.Application.Info.DirectoryPath & "\..\..\..\..\..\media\")

        Creating()

        Loading()

        Updating()

        Rendering()
    End Sub

    Private Sub Creating()
        'creation de la scene. A noter que chaque nom de classes et composants doivent être unique
        myScene = New Scene.TZScene("myScene")
        myScene.Lighting = False
        'création d'une caméra
        myCamera = New Tzu3D.Component.TZCamera("myCamera")

        'avec un champ de vision de 75°, d'un rapport d'écran en 16/9 et dont le rendu doit s'effectuer à une distance commprise entre 0.1 et 50
        myCamera.Perspective(75, 16 / 9, 0.1, 50)

        'On indique que l'on souhaite une projection en perspective
        myCamera.Projection_Mode = eCameraProjection.Perspective

        'Et d'un mode de déplacement à la première personnes
        myCamera.Set_Mode(eCameraMode.First_Person)

        'Notre composant camera est créé, il faut maintenant créé un acteur qui va porter cette camera
        myCameraActor = New Scene.TZActor("myCameraActor")

        'On positionne notre cameraman legerement en hauteur et en retrait
        myCameraActor.Position = New Vector3(0, 4, -4)

        'Et finalement on donne notre composant camera à cet acteur
        myCameraActor.Add_Component(myCamera)

        'Notre cameraman est prèt, il peut entrer en scene
        myScene.Add(myCameraActor)

        'création d'un mesh vide puis on lui indique de créé un cube de 3x3x3
        myMesh = New Tzu3D.Component.TZMesh("cube")
        myMesh.Create_Cube(3, 3, 3)

        'On récupère le materiau de notre mesh et lui indiquons une couleur diffuse blanche
        Dim MeshMaterial As Texture.TZMaterial = myMesh.Material(0)
        MeshMaterial.ColorDiffuse = Color.White

        'Nous appliquons également une texture de brick en créant une texture à partir d'une image stocké dans le dossier media
        Dim Texture_Diffuse As Texture.TZTexture = New Texture.TZTexture("Brick_Wall_017_basecolor.jpg", "DiffuseMap")
        MeshMaterial.Texture_Diffuse = Texture_Diffuse

        'Notre materiau est modifié, il sufffit de le redonner à notre cube
        myMesh.Material(0) = MeshMaterial

        'Création d'un nouvel acteur qui va enfiler son costume en forme de cube et se positionner au centre de la scene
        myActor = New Scene.TZActor("myActor")
        myActor.Position = New Vector3(0, 0, 0)
        myActor.Add_Component(myMesh)

        'notre acteur en forme de cube peut lui aussi entrer en scene
        myScene.Add(myActor)
    End Sub

    Private Sub Loading()
        'chargement de la scene
        myScene.Load()

        'chargement du moteur
        Engine.Load()
    End Sub

    Private Sub Updating()
        'On indique au cameraman de regarder au centre de la scene
        myCamera.LookAt(New Vector3(0, 0, 0))
    End Sub

    Private Sub Rendering()
        Do While Engine.DoRender
            Engine.Render_Begin(Color.BlueViolet)

            Controller()

            'Process: détermine les objets qui seront visible à l'écran ainsi que divers autres traitement (à n'appeler qu'une seul fois par cycle)
            myScene.Process()

            myScene.Render()

            Engine.Render_End()

            Application.DoEvents()
        Loop

        Engine.Dispose()

        Application.Exit()
    End Sub

    Private Sub Controller()
        'rotation du cube
        myActor.Rotate_StepY(60 * Engine.DeltaTime)
    End Sub

    Private Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        If Engine.DoRender Then
            Engine.Quit()
            e.Cancel = True
        End If
    End Sub

    'si la fenêtre est redimensionnée, alors on redimensionne aussi les composants internes au moteur afin d'éviter d'avoir un affichage étiré ou compressé
    Private Sub Form1_ResizeEnd(sender As Object, e As EventArgs) Handles Me.ResizeEnd
        If Engine Is Nothing Then Exit Sub
        Engine.Resize_Windows()
    End Sub

    'même chose si la fenêtre est maximisée
    Dim OldWindowState As FormWindowState
    Private Sub Form1_SizeChanged(sender As Object, e As EventArgs) Handles Me.SizeChanged
        If Engine Is Nothing Then Exit Sub 
        If Me.WindowState = FormWindowState.Maximized OrElse Me.WindowState = FormWindowState.Normal Then
            If OldWindowState <> Me.WindowState Then
                Engine.Resize_Windows()
                OldWindowState = Me.WindowState
            End If
        End If
    End Sub
End Class
